package ch.rakudave.jnetmap.controller;

import java.awt.Component;
import java.awt.Toolkit;
import java.io.File;
import java.io.FileInputStream;
import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ScheduledThreadPoolExecutor;

import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.filechooser.FileNameExtensionFilter;

import net.xeoh.plugins.base.PluginManager;
import net.xeoh.plugins.base.impl.PluginManagerFactory;
import net.xeoh.plugins.base.options.addpluginsfrom.OptionLazyActivation;
import ch.rakudave.jnetmap.model.Map;
import ch.rakudave.jnetmap.model.MapEvent.Type;
import ch.rakudave.jnetmap.model.device.Device;
import ch.rakudave.jnetmap.util.Crypto;
import ch.rakudave.jnetmap.util.IO;
import ch.rakudave.jnetmap.util.Icons;
import ch.rakudave.jnetmap.util.Lang;
import ch.rakudave.jnetmap.util.Settings;
import ch.rakudave.jnetmap.util.SwingHelper;
import ch.rakudave.jnetmap.util.XStreamHelper;
import ch.rakudave.jnetmap.util.logging.FileAppender;
import ch.rakudave.jnetmap.util.logging.Logger;
import ch.rakudave.jnetmap.util.logging.Logger.Level;
import ch.rakudave.jnetmap.view.IView;
import ch.rakudave.jnetmap.view.MapView;
import ch.rakudave.jnetmap.view.components.SplashScreen;
import ch.rakudave.jnetmap.view.preferences.Preferences;

/**
 * @author rakudave
 */
public class Controller {
	public static final String version = "0.5.4";
	private static IView view;
	protected static Preferences preferences;
	private static List<Map> maps;
	private static Map currentMap;
	public static PluginManager pm;
	
	public static void main(String[] args) {
		try {
			InstanceDetector.delegate(args);
			try { // Set native look-and-feel
				if (IO.isOSX) System.setProperty("apple.laf.useScreenMenuBar", "true");
				UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
				Class<?> xtoolkit = Toolkit.getDefaultToolkit().getClass();
				if (xtoolkit.getName().equals("sun.awt.X11.XToolkit")) {
					final Field awtAppClassName = xtoolkit.getDeclaredField("awtAppClassName");
					awtAppClassName.setAccessible(true);
					awtAppClassName.set(null, "jNetMap");
				}
			} catch (Exception e) {
				Logger.warn("Unable to set 'System Look-And-Feel', falling back to Swing");
			}
			new Controller(args);
		} catch (Throwable t) {
			Logger.error("Unhandled exception", t);
		}
	}

	private Controller(String[] args) {
		SplashScreen.create();
		SplashScreen.setProgress(0, "Initial setup...");
		Scheduler.executor = new ScheduledThreadPoolExecutor(3);
		maps = new LinkedList<Map>();
		IO.installUserFiles();
		SplashScreen.setProgress(15, "Loading preferences...");
		Settings.load();
		Logger.addAppender(new FileAppender(Level.valueOf(Settings.get("logger.FileAppender", "ERROR"))));
			Logger.debug("Java: "+System.getProperty("java.version")+" "+System.getProperty("sun.arch.data.model")+"bit by "+System.getProperty("java.vendor"));
			Logger.debug("OS: "+System.getProperty("os.name")+" "+System.getProperty("os.version")+" "+System.getProperty("os.arch"));
		RecentlyOpened.load();
		SplashScreen.setProgress(30, "Loading language...");
		Lang.load();
		SplashScreen.setProgress(45, "Loading icons...");
		Icons.load();
		SplashScreen.setProgress(60, "Loading plugins...");
		pm = PluginManagerFactory.createPluginManager();
		pm.addPluginsFrom(IO.pluginDir.toURI(), new OptionLazyActivation());
		SplashScreen.setProgress(75, "Loading GUI...");
		view = new MapView();
		preferences = new Preferences(view);
		SplashScreen.setProgress(90, "Opening files...");
		open(args);
		if (args.length == 0 && Settings.getBoolean("maps.restore", false)) {
			int count = Settings.getInt("maps.count", 0);
			if (count != 0) {
				for (int i = 0; i < count; i++) {
					open(getMapFromFile(Settings.get("maps."+i, ""), null));
				}
			}
		}
		SplashScreen.dispose();
	}

	public static void open(String[] args) {
		if (args.length == 0) return;
		Logger.info("Opening " + Arrays.toString(args));
		for (String file : args) {
			if ("--new".equals(file) || "-n".equals(file)) {
				Controller.getView().openMap(new Map());
			} else if ("--open".equals(file) || "-o".equals(file)) {
				Object[] o = SwingHelper.openDialog((Component) Controller.getView(), new FileNameExtensionFilter("jNetMap file *.jnm", "jnm"), true);
				if (o == null) return;
				File f = (File) o[0];
				if (f == null) return;
				open(getMapFromFile(f.getAbsolutePath(), null));
			} else if (!file.isEmpty()) {
				open(getMapFromFile(file, null));
			}
		}
		view.toFront();
	}
	
	public static boolean open(Map map) {
		if (map == null) return false;
		Logger.info("Opening '" + map.getFileName() + "'");
		if (maps.contains(map)) {
			Logger.warn("Map is already open: " + map.getFileName());
			return true;
		}
		try {
			view.openMap(map);
			setCurrentMap(map);
			return true;
		} catch (Exception e) {
			Logger.error("Unable to open " + map.getFileName(), e);
			JOptionPane.showMessageDialog((Component) Controller.view, Lang.get("map.fail.open"), map.getFileName(), JOptionPane.ERROR_MESSAGE);
			return false;
		}
	}
	
	public static Map getMapFromFile(String filePath, String password) {
		try {
			File mapFile = new File(filePath);
			String xml = IO.getString(new FileInputStream(mapFile));
			if (xml == null) return null;
			if (!xml.startsWith("<Map>") && (password == null || password.isEmpty())) {
				password = SwingHelper.passwordPrompt();
				if (password == null) return null;
			}
			if (password != null && !password.isEmpty()) {
				xml = Crypto.decrypt(xml, password);
			}
			Map m = (Map) XStreamHelper.getXStream().fromXML(xml);
			m.setFile(mapFile);
			m.setPassword(password);
			RecentlyOpened.put(mapFile.getAbsolutePath());
			return m;
		} catch (Exception e) {
			Logger.error("Could not open file '" + filePath + "'", e);
			return null;
		}
	}
	
	public static boolean close(Map map) {
		if (map == null || !maps.contains(map)) return true;
		Logger.trace("Closing map "+map);
		if (!map.isSaved()){
			String msg = Lang.get("message.unsaved").replaceAll("%name%", map.getFileName());
			int i = JOptionPane.showConfirmDialog((Component) view, msg, 
					map.getFilePath(), JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
			switch (i) {
				case 2: return false;
				case 0:
					setStatusUnknownAndSave(map);
					break;
				default: break;
			}
		} else {
			setStatusUnknownAndSave(map);
		}
		maps.remove(map);
		return true;
	}
	
	private static void setStatusUnknownAndSave(Map map) {
		for (Device d : map.getVertices()) d.addStatusUnknownToHistory();
		setCurrentMap(map);
		Actions.save().actionPerformed(null);
	}

	public static void setCurrentMap(Map map) {
		if (!maps.contains(currentMap)) {
			maps.add(map);
			StatusUpdater.addMap(map);
		}
		currentMap = map;
		view.setWindowTitle(map.getFilePath());
	}

	public static Map getCurrentMap() {
		return currentMap;
	}

	public static boolean closeAllMaps() {
		List<Map> tempMaps = new LinkedList<Map>(maps);
		int i = 0;
		for (Map m : tempMaps) {
			if (!close(m)) return false;
			if (!m.getFileName().equals(Lang.getNoHTML("map.newmap"))) 
				Settings.put("maps."+i++, m.getFilePath());
		}
		Settings.put("maps.count", i);
		return true;
	}

	public static IView getView() {
		return view;
	}
	
	public static void refreshAll() {
		for (Map m : maps) m.refreshView(Type.SETTINGS_CHANGED);
	}
}
