package ch.rakudave.jnetmap.net.status;

import java.io.IOException;
import java.net.InetAddress;

import ch.rakudave.jnetmap.util.Settings;
import ch.rakudave.jnetmap.util.logging.Logger;

import com.thoughtworks.xstream.annotations.XStreamAlias;

/**
 * @author rakudave
 */

@XStreamAlias("IsReachable")
public class IsReachable implements PingMethod {
	private static IsReachable instance;
	private static Object lock = new Object();
	
	public static IsReachable getInstance() {
		if (instance == null) {
			synchronized (lock) {
				if (instance == null) {
					instance = new IsReachable();
				}
			}
		}
		return instance;
	}
	
	private IsReachable() {}

	@Override
	public Status getStatus(InetAddress address) {
		try {
			if (address.isReachable(Settings.getInt("isreachable_timeout", Settings.getInt("ping.timeout", 5000)))) {
				Logger.trace(address + " is up");
				return Status.UP;
			} else {
				Logger.trace(address + " is down");
				return Status.DOWN;
			}
		} catch (IOException e) {
			Logger.trace(address + " is unknown");
			return Status.UNKNOWN;
		}
	}
	
	@Override
	public String toString() {
		return "Java Ping";
	}

}
